/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: LayoutHelper.cpp,v 2.4 1999/10/16 01:45:13 jeff Exp $

*******************************************************************************/


#include <iostream.h>

#include <qapp.h>
#include <qlayout.h>

#include "layout/LayoutHelper.h"

#include "WidgetLayout.h"


LayoutHelper::LayoutHelper( WidgetLayout&	widgetLayout )
    :
    isInConstructor( TRUE ),
    myWidgetLayout( widgetLayout ),
    myLayoutWindow( NULL ),
    myLayout( NULL ),
    myLayoutTree( NULL ),
    myWindowFrame( *this, myLayoutTree ),
    myLayoutItemInConstruction( NULL ),
    isInRestoration( FALSE )
{
    isInConstructor = FALSE;

    connect( &myWindowFrame, SIGNAL(ContentsChanged()),
	     &myWidgetLayout, SIGNAL(Changed()) );

    connect( &myWindowFrame, SIGNAL(Closed()),
	     &myWidgetLayout, SIGNAL(LayoutWndClosed()) );
}


LayoutHelper::~LayoutHelper()
{
    // Deleting the layout window will delete myLayout.
}


// Delete the specified dlgWidget from the layout
void LayoutHelper::DeleteWidget( DlgWidget*	dlgWidget )
{
    myWindowFrame.GetRootTreeItem()->DeleteWidget( dlgWidget );

    RedrawLayout();
}


//
// Redraw the layout
//

void LayoutHelper::RedrawLayout()
{
    if( myLayoutWindow )
    {
	QSize frameSize = myLayoutWindow->size();
	
	delete myLayout;
    
	myLayout = 
	    myWindowFrame.GetRootTreeItem()->RecreateLayout( myLayoutWindow );
	if( myLayout )
	{
	    myLayout->activate();
	}

	// Need this so events get processed before setting the size.
	qApp->processEvents();

	myLayoutWindow->setFixedSize( frameSize );

	if( !isInConstructor )
	{
	    myWidgetLayout.UpdateNoMoveWidgets();
	}
    }
}


//
// Load and Save the layout
//

// Restore a new layout item
void LayoutHelper::RestoreNewLayoutItem( QString&	theItemName )
{
    isInRestoration = TRUE;

    // break item name into type and id
    int pos = theItemName.find( '-' );
    QString type = theItemName.left( pos );
    QString id = theItemName.mid( pos + 1, theItemName.length() - pos );

    LayoutItem::LayoutItemType itemType = LayoutItem::String2Type( type );
    LayoutItem* item = 0;
    
    if( itemType != LayoutItem::LI_None )
    {
	if( id == "1" )
	{
	    // Insert the new item into the root of the tree.
	    item = myWindowFrame.InsertRootLayout( itemType );
	}
	else
	{
	    // Strip off the first two chars, the '1.', of the id.
	    id = id.mid( 2, id.length() - 2 );

	    // Insert the new item into its place in the tree.
	    item = 
		myWindowFrame.GetRootTreeItem()->InsertNewLayoutItem( itemType,
								      id );
	}

	myLayoutItemInConstruction = item;
    }
    else
    {
        cerr << tr( "Unknown layout item type: %1" ).arg( type ) << endl;
    }
}


// Restore a single key/value pair for a layout item
void LayoutHelper::RestoreLayoutItemKeyValue( QString&	theKey,
					      QString&	theValue )
{
    const DlgWidget* widget = 0;

    if( ! theValue.isEmpty() )
      widget = myWidgetLayout.GetWidget( theValue );

    if( myLayoutItemInConstruction &&	
	!myLayoutItemInConstruction->RestoreKeyValue( theKey, 
						      theValue, 
						      widget ) )
    {
        cerr << tr( "Unknown key value Pair: < %1 : %2 >" ).
            arg( theKey ).arg( theValue ) << endl;
    }
}


// Constructing a new layout item is finished
void LayoutHelper::RestoreEndLayoutItem()
{
    myLayoutItemInConstruction = NULL;
}


// All the layout contents have been restored
void LayoutHelper::ContentsRestored()
{
    isInRestoration = FALSE;

    myLayoutTree->repaint();

    RedrawLayout();
}


// Save the layout in the given stream
void LayoutHelper::SaveLayout( QTextStream&	theStream ) const
{
    myWindowFrame.GetRootTreeItem()->SaveContents( theStream, "1" );
}


//
// Generate source for the layout
//

// generate the source for the layout of the window
void LayoutHelper::GenerateLayoutSource( QTextStream&	theStream ) const
{
    myWindowFrame.GetRootTreeItem()->GenerateSource( theStream );
}


void LayoutHelper::SelectWidget( 
    const DlgWidget* widget 
    )
{
    myWidgetLayout.ClearSelection();
    myWidgetLayout.AddToSelection( widget );
}


void LayoutHelper::ShowWidgetProperties(
    const DlgWidget*    widget
    )
{
    if( !myWidgetLayout.IsSelected( widget ) )
    {
        myWidgetLayout.ClearSelection();
        myWidgetLayout.AddToSelection( widget );
    }
    
    myWidgetLayout.SelectedWidgetProperties();
}

