/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: LayoutHelper.h,v 2.3 1999/10/16 01:45:14 jeff Exp $

*******************************************************************************/


#ifndef LayoutHelper_included
#define LayoutHelper_included

#include <qlistview.h>
#include <qobject.h>
#include <qstring.h>
#include <qtstream.h>
#include <qwidget.h>

#include "layout/LayoutWnd.h"

class WidgetLayout;
class DlgWidget;
class LayoutItem;
class QLayout;


class LayoutHelper : public QObject
{
    Q_OBJECT

public: // Methods

    LayoutHelper( WidgetLayout&	widgetLayout );
    // Constructor

    ~LayoutHelper();
    // Destructor

    void SetLayoutWindow( QWidget*	layoutWindow );
    // Set the window to use for the layout

    void Show( bool	show );
    // Show or hide the layout manager window

    void SetCaption(
	const QString&	theCaption
	);
    // Set the caption for the layout window

    void DeleteWidget( DlgWidget*	dlgWidget );
    // Delete the specified dlgWidget from the layout

    void SelectWidget( const DlgWidget* widget );
    // Select the given widget in the widget layout

    void ShowWidgetProperties( const DlgWidget* widget );
    // Show the properties for the widget

    const WidgetLayout& GetWidgetLayout( void ) const;
    // Get the widget layout for the dialog

    //
    // Redraw the layout
    //
    
    void RedrawLayout();
    // Redraw the entire layout.

    //
    // Load and Save the layout
    //

    void SaveLayout( QTextStream&	theStream ) const;
    // Save the layout in the given stream
    

    //
    // Generate source for the layout
    //

    void GenerateLayoutSource( QTextStream&	theStream ) const;
    // generate the source for the layout of the window


public slots: // Slots
    
    //
    // Load and Save the layout
    //

    void RestoreNewLayoutItem( QString&	theItemName );
    // Restore a new layout item

    void RestoreLayoutItemKeyValue( QString&	theKey,
				    QString&	theValue );
    // Restore a single key/value pair for a layout item

    void RestoreEndLayoutItem();
    // Constructing a new layout item is finished

    void ContentsRestored();
    // All the layout contents have been restored

private: // Methods

    LayoutHelper( const LayoutHelper& );
    // Copy constructor

    LayoutHelper& operator=( const LayoutHelper& );
    // = operator


private: // Attributes

    bool	isInConstructor;
    // Is the class in its constructor.

    WidgetLayout&	myWidgetLayout;
    // The WidgetLayout this class helps

    QWidget*	myLayoutWindow;
    // Window used for the layout

    QLayout*	myLayout;
    // The layout used on myLayoutWindow.

    QListView*	myLayoutTree;
    // Tree for displaying the layout, deleted by LayoutWnd

    LayoutWnd	myWindowFrame;
    // Frame for displaying layout tree

    LayoutItem* myLayoutItemInConstruction;
    // The layout item being constructed

    bool	isInRestoration;
    // Are the contents being restored

    static int layoutNumber;
    // Numbering all the Layouts to allow automatic name generation

};


inline void LayoutHelper::SetLayoutWindow( QWidget*	layoutWindow )
{
    myLayoutWindow = layoutWindow;
}


inline void LayoutHelper::Show( bool	show )
{
    if( show )
    {
	myWindowFrame.show();
    }
    else
    {
	myWindowFrame.hide();
    }
}


inline void LayoutHelper::SetCaption(
    const QString&	theCaption
    )
{
    myWindowFrame.setCaption( tr( "Qt Architect Layout: %1" ).arg( theCaption ) );
    myWindowFrame.setIconText( myLayoutWindow->caption() );
}


inline const WidgetLayout& LayoutHelper::GetWidgetLayout( void ) const
{
    return myWidgetLayout;
}


#endif // LayoutHelper_included
