/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgComboBox.cpp,v 2.5 1999/10/16 01:50:32 jeff Exp $

*******************************************************************************/


#include "widget/DlgComboBox.h"

#include "prop/ComboBoxPage.h"
#include "prop/WidgetBitmapPage.h"

#include "GuiTools.h"

#define Inherited DlgWidget

QStrList DlgComboBox::itsSignals;
int DlgComboBox::itsNextWidgetNum = 1;

DlgComboBox::DlgComboBox() :
    Inherited( DlgWidget::DW_ComboBox ),
    myComboType( CT_ReadOnly ),
    isAutoCompletion( FALSE )
{
}


DlgComboBox::DlgComboBox(
    DlgWidget::Type	theType
    ) :
    Inherited( theType ),
    myComboType( CT_ReadOnly ),
    isAutoCompletion( FALSE )
{
}


DlgComboBox::~DlgComboBox()
{
}


void DlgComboBox::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new ComboBoxPage( this, parent, "ComboBox" ) );

    labels.append( tr( "Combo Box" ) );
}


void DlgComboBox::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "ComboBox {" << endl;
    }
  
    QComboBox* widget = GetComboBox();

    stream << saveIndent << "Style {";
    switch( myComboType )
    {
	case CT_ReadOnly:
	{
	    stream << "ReadOnly";
	    break;
	}
	case CT_ReadWrite:
	{
	    stream << "ReadWrite";
	    break;
	}
	case CT_Motif1x:
	{
	    stream << "Motif1x";
	    break;
	}
    }
    stream << "}" << endl;

    stream << saveIndent <<
	"SizeLimit {" << widget->sizeLimit() << "}" << endl;
    
    stream << saveIndent << "Policy {";
    switch( widget->insertionPolicy() )
    {
	case QComboBox::NoInsertion:
	{
	    stream << "NoInsertion";
	    break;
	}
	case QComboBox::AtTop:
	{
	    stream << "AtTop";
	    break;
	}
	case QComboBox::AtCurrent:
	{
	    stream << "AtCurrent";
	    break;
	}
	case QComboBox::AtBottom:
	{
	    stream << "AtBottom";
	    break;
	}
	case QComboBox::AfterCurrent:
	{
	    stream << "AfterCurrent";
	    break;
	}
	case QComboBox::BeforeCurrent:
	{
	    stream << "BeforeCurrent";
	    break;
	}
    }
    stream << "}" << endl;

    stream << saveIndent << "AutoResize {";
    SaveBool( stream, widget->autoResize() );
    stream << "}" << endl;
    
    for( int i = 0; i < widget->count(); ++i )
    {
	stream << saveIndent << "Item {" << widget->text( i ) << "}" << endl;
    }

    stream << saveIndent << 
	"MaxCount {" << widget->maxCount() << "}" << endl;
    
    stream << saveIndent << "AutoCompletion {";
    SaveBool( stream, isAutoCompletion );
    stream << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgComboBox::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QComboBox* widget = GetComboBox();

    // Compare to known keys
    if( key == "Style" )
    {
	if( value == "ReadOnly" )
	{
	    myComboType = CT_ReadOnly;
	}
	else if( value == "ReadWrite" )
	{
	    myComboType = CT_ReadWrite;
	}
	else if( value == "Motif1x" )
	{
	    myComboType = CT_Motif1x;
	}
	else
	{
	    retVal = FALSE;
	}
    } 
    else if( key == "Item" )
    {
	widget->insertItem( value );
    } 
    else if( key == "SizeLimit" )
    {
	widget->setSizeLimit( value.toInt() );
    }
    else if( key == "Policy" )
    {
	if( value == "NoInsertion" )
	{
	    widget->setInsertionPolicy( QComboBox::NoInsertion );
	}
	else if( value == "AtTop" )
	{
	    widget->setInsertionPolicy( QComboBox::AtTop );
	}
	else if( value == "AtCurrent" )
	{
	    widget->setInsertionPolicy( QComboBox::AtCurrent );
	}
	else if( value == "AtBottom" )
	{
	    widget->setInsertionPolicy( QComboBox::AtBottom );
	}
	else if( value == "AfterCurrent" )
	{
	    widget->setInsertionPolicy( QComboBox::AfterCurrent );
	}
	else if( value == "BeforeCurrent" )
	{
	    widget->setInsertionPolicy( QComboBox::BeforeCurrent );
	}
	else
	{
	    retVal = FALSE;
	}
    }
    else if( key == "AutoResize" )
    {
	widget->setAutoResize( (value == "TRUE") );
    }
    else if( key == "MaxCount" )
    {
	widget->setMaxCount( value.toInt() );
    }
    else if( key == "AutoCompletion" )
    {
	isAutoCompletion = (value == "TRUE");
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgComboBox::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QComboBox* widget = GetComboBox();

    switch( myComboType )
    {
	case CT_ReadWrite:
	{
	    stream << varName << "setInsertionPolicy( QComboBox::";
	    switch( widget->insertionPolicy() )
	    {
		case QComboBox::NoInsertion:
		{
		    stream << "NoInsertion";
		    break;
		}
		case QComboBox::AtTop:
		{
		    stream << "AtTop";
		    break;
		}
		case QComboBox::AtCurrent:
		{
		    stream << "AtCurrent";
		    break;
		}
		case QComboBox::AtBottom:
		{
		    stream << "AtBottom";
		    break;
		}
		case QComboBox::AfterCurrent:
		{
		    stream << "AfterCurrent";
		    break;
		}
		case QComboBox::BeforeCurrent:
		{
		    stream << "BeforeCurrent";
		    break;
		}
	    }
	    stream << " );" << endl;
	    
	    // fall through
	}
	case CT_ReadOnly:
	{
	    stream << varName << "setSizeLimit( " << widget->sizeLimit() <<
		" );" << endl;
	    break;
	}
	case CT_Motif1x:
	{
	    break; // do nothing
	}
    }

    stream << varName << "setAutoResize( " <<
	(widget->autoResize() ? "TRUE" : "FALSE") << " );" << endl;
    
    for( int i = 0; i < widget->count(); i++ )
    {
	stream << varName << "insertItem( ";
        GenerateText( widget->text( i ), widgetLayout, stream );
        stream << " );" << endl;
    }

    stream << varName << "setMaxCount( " << widget->maxCount() << " );" << endl;

    stream << varName << "setAutoCompletion( " <<
	(isAutoCompletion ? "TRUE" : "FALSE") << " );" << endl;
}


void DlgComboBox::GenerateCtor(
    QTextStream&	stream,
    const QString&	parentName
    )
{
    QString name;
    const char* cp = GetWidget()->name();
    if( cp )
    {
	name = "\"";
	name += cp;
	name += "\"";
    }
    else
    {
	name = "NULL";
    }

    switch( myComboType )
    {
	case CT_ReadOnly:
	{
	    stream << "FALSE, ";
	    break;
	}
	case CT_ReadWrite:
	{
	    stream << "TRUE, ";
	    break;
	}
	case CT_Motif1x:
	{
	    break; // do nothing
	}
    }

    stream << parentName << ", " << name;
}


void DlgComboBox::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgComboBox::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "activated(int)" );
	itsSignals.append( "activated(const QString&)" );
	itsSignals.append( "highlighted(int)" );
	itsSignals.append( "highlighted(const QString&)" );
	itsSignals.append( "textChanged(const QString&)" );
    }

    return itsSignals;
}


void DlgComboBox::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QComboBox* widget = 
	new QComboBox( FALSE,
		       theParent,
		       CreateName( "ComboBox", 
				   theName, 
				   itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
