/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgSlider.cpp,v 2.3 1999/10/16 01:51:13 jeff Exp $

*******************************************************************************/


#include "widget/DlgSlider.h"

#include "prop/SliderPage.h"

#include "GuiTools.h"

#define Inherited DlgWidget

QStrList DlgSlider::itsSignals;
int DlgSlider::itsNextWidgetNum = 1;

DlgSlider::DlgSlider() :
    Inherited( DlgWidget::DW_Slider )
{
}


DlgSlider::DlgSlider(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgSlider::~DlgSlider()
{
}


void DlgSlider::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new SliderPage( this, parent, "Slider" ) );

    labels.append( tr( "Slider" ) );
}


void DlgSlider::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "Slider {" << endl;
    }
  
    QSlider* widget = GetSlider();

    stream << saveIndent << "Orientation {";
    if( widget->orientation() == QSlider::Horizontal )
    {
	stream << "Horizontal";
    }
    else
    {
	stream << "Vertical";
    }
    stream << "}" << endl;
    
    stream << saveIndent << "MinValue {" << widget->minValue() << "}" << endl;
    stream << saveIndent << "MaxValue {" << widget->maxValue() << "}" << endl;
    stream << saveIndent << "Initial {" << widget->value() << "}" << endl;
    stream << saveIndent << "LineStep {" << widget->lineStep() << "}" << endl;
    stream << saveIndent << "PageStep {" << widget->pageStep() << "}" << endl;
    
    stream << saveIndent << "Tracking {";
    SaveBool( stream, widget->tracking() );
    stream << "}" << endl;
    
    stream << saveIndent << "Tickmarks {" <<
	GuiTools::QSliderTickSettingStr( widget->orientation(),	
					 widget->tickmarks() ) << "}" << endl;
    
    stream << saveIndent << 
	"TickInterval {" << widget->tickInterval() << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgSlider::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QSlider* widget = GetSlider();

    // Compare to known keys
    if( key == "Orientation" )
    {
	if( value == "Horizontal" )
	{
	    widget->setOrientation( QSlider::Horizontal );
	}
	else
	{
	    widget->setOrientation( QSlider::Vertical );
	}
    } 
    else if( key == "MinValue" )
    {
	widget->setRange( value.toInt(), widget->maxValue() );
    }
    else if( key == "MaxValue" )
    {
	widget->setRange( widget->minValue(), value.toInt() );
    }
    else if( key == "Initial" )
    {
	widget->setValue( value.toInt() );
    }
    else if( key == "LineStep" )
    {
	widget->setSteps( value.toInt(), widget->pageStep() );
    }
    else if( key == "PageStep" )
    {
	widget->setSteps( widget->lineStep(), value.toInt() );
    }
    else if( key == "Tracking" )
    {
	widget->setTracking( (value == "TRUE") );
    }
    else if( key == "Tickmarks" )
    {
	widget->setTickmarks( GuiTools::QSliderTickSetting( value ) );
    }
    else if( key == "TickInterval" )
    {
	widget->setTickInterval( value.toInt() );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgSlider::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QSlider* widget = GetSlider();

    stream << varName << "setOrientation( QSlider::" <<
	((widget->orientation() == QSlider::Horizontal) ? "Horizontal" : "Vertical") <<
	" );" << endl;
    
    stream << varName << "setRange( " <<
	widget->minValue() << ", " << widget->maxValue() << " );" << endl;
    
    stream << varName << "setSteps( " <<
	widget->lineStep() << ", " << widget->pageStep() << " );" << endl;
    
    stream << varName << 
	"setValue( " << widget->value() << " );" << endl;
    
    stream << varName << "setTracking( " <<
	(widget->tracking() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName << "setTickmarks( QSlider::" <<
	GuiTools::QSliderTickSettingStr( widget->orientation(),
					 widget->tickmarks() ) << " );" << endl;
    
    stream << varName <<
	"setTickInterval( " << widget->tickInterval() << " );" << endl;
}


void DlgSlider::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgSlider::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "valueChanged(int)" );
	itsSignals.append( "sliderPressed()" );
	itsSignals.append( "sliderMoved(int)" );
	itsSignals.append( "sliderReleased()" );
    }

    return itsSignals;
}


void DlgSlider::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QSlider* widget = 
	new QSlider( QSlider::Horizontal,
		     theParent,
		     CreateName( "Slider", 
				 theName, 
				 itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
